@@ -7,6 +7,7 @@
 #include <QCheckBox>
 #include <QComboBox>
 #include <QCompleter>
+#include <QFileDialog>
 
 #include <QPushButton>
 #include <QScrollArea>
@@ -15,6 +16,7 @@
 #include <QMouseEvent>
 #include <QSpinBox>
 #include <QTableView>
+#include <QTableWidget>
 #include <QtWidgets/QColorDialog>
 
 OBS_DECLARE_MODULE()
@@ -65,6 +67,38 @@ static void load_transition_matrix(obs_data_t *obj)
 	obs_data_array_release(transitions);
 }
 
+static void load_transitions(obs_data_t *obj)
+{
+	obs_data_array_t *transitions =
+		obs_data_get_array(obj, "transitions");
+	if (!transitions)
+		return;
+	const size_t count =
+		obs_data_array_count(transitions);
+	for (size_t i = 0; i < count; i++) {
+		obs_data_t *transition =
+			obs_data_array_item(transitions,
+			                    i);
+		string fromScene = obs_data_get_string(
+			transition, "from_scene");
+		string toScene = obs_data_get_string(
+			transition, "to_scene");
+		string transitionName =
+			obs_data_get_string(
+				transition,
+				"transition");
+		const uint32_t duration =
+			obs_data_get_int(transition,
+			                 "duration");
+		transition_table[fromScene][toScene]
+			.transition = transitionName;
+		transition_table[fromScene][toScene]
+			.duration = duration;
+		obs_data_release(transition);
+	}
+	obs_data_array_release(transitions);
+}
+
 static void frontend_save_load(obs_data_t *save_data, bool saving, void *)
 {
 	if (saving) {
@@ -85,6 +119,7 @@ static void frontend_save_load(obs_data_t *save_data, bool saving, void *)
 						 it2.second.duration);
 				obs_data_array_push_back(transitions,
 							 transition);
+				obs_data_release(transition);
 			}
 		}
 		obs_data_set_array(obj, "transitions", transitions);
@@ -125,36 +160,7 @@ static void frontend_save_load(obs_data_t *save_data, bool saving, void *)
 			obs_hotkey_pair_load(transition_table_hotkey, eh, dh);
 			obs_data_array_release(eh);
 			obs_data_array_release(dh);
-			obs_data_array_t *transitions =
-				obs_data_get_array(obj, "transitions");
-			if (transitions) {
-				size_t count =
-					obs_data_array_count(transitions);
-				for (size_t i = 0; i < count; i++) {
-					obs_data_t *transition =
-						obs_data_array_item(transitions,
-								    i);
-					string fromScene = obs_data_get_string(
-						transition, "from_scene");
-					string toScene = obs_data_get_string(
-						transition, "to_scene");
-					string transitionName =
-						obs_data_get_string(
-							transition,
-							"transition");
-					bool custom_duration = obs_data_get_bool(
-						transition, "custom_duration");
-					const uint32_t duration =
-						obs_data_get_int(transition,
-								 "duration");
-					transition_table[fromScene][toScene]
-						.transition = transitionName;
-					transition_table[fromScene][toScene]
-						.duration = duration;
-					obs_data_release(transition);
-				}
-				obs_data_array_release(transitions);
-			}
+			load_transitions(obj);
 			obs_data_release(obj);
 		} else {
 			obj = obs_data_get_obj(save_data,
@@ -266,6 +272,7 @@ static void frontend_event(enum obs_frontend_event event, void *)
 
 static void source_rename(void *data, calldata_t *call_data)
 {
+	UNUSED_PARAMETER(data);
 	string new_name = calldata_string(call_data, "new_name");
 	string prev_name = calldata_string(call_data, "prev_name");
 	auto it = transition_table.find(prev_name);
@@ -285,6 +292,9 @@ static void source_rename(void *data, calldata_t *call_data)
 bool enable_hotkey(void *data, obs_hotkey_pair_id id, obs_hotkey_t *hotkey,
 		   bool pressed)
 {
+	UNUSED_PARAMETER(data);
+	UNUSED_PARAMETER(id);
+	UNUSED_PARAMETER(hotkey);
 	if (!transition_table_enabled && pressed) {
 		transition_table_enabled = true;
 		set_transition_overrides();
@@ -296,6 +306,9 @@ bool enable_hotkey(void *data, obs_hotkey_pair_id id, obs_hotkey_t *hotkey,
 bool disable_hotkey(void *data, obs_hotkey_pair_id id, obs_hotkey_t *hotkey,
 		    bool pressed)
 {
+	UNUSED_PARAMETER(data);
+	UNUSED_PARAMETER(id);
+	UNUSED_PARAMETER(hotkey);
 	if (transition_table_enabled && pressed) {
 		transition_table_enabled = false;
 		clear_transition_overrides();
@@ -457,16 +470,133 @@ TransitionTableDialog::TransitionTableDialog(QMainWindow *parent)
 	scrollArea->setWidget(widget);
 	scrollArea->setWidgetResizable(true);
 
-	QPushButton *closeButton = new QPushButton(obs_module_text("Close"));
-	QPushButton *deleteButton = new QPushButton(obs_module_text("Delete"));
+	QPushButton *closeButton =
+		new QPushButton(QString::fromUtf8(obs_module_text("Close")));
+	QPushButton *exportButton =
+		new QPushButton(QString::fromUtf8(obs_module_text("Export")));
+	QPushButton *matrixButton =
+		new QPushButton(QString::fromUtf8(obs_module_text("Matrix")));
+	QPushButton *importButton =
+		new QPushButton(QString::fromUtf8(obs_module_text("Import")));
+	QPushButton *deleteButton =
+		new QPushButton(QString::fromUtf8(obs_module_text("Delete")));
 
 	QHBoxLayout *bottomLayout = new QHBoxLayout;
-	bottomLayout->addWidget(deleteButton, 0, Qt::AlignLeft);
+	bottomLayout->addWidget(
+		new QLabel(
+			"<a href=\"https://obsproject.com/forum/resources/transition-table.1174/\">Transition Table</a> (" PROJECT_VERSION
+			") by <a href=\"https://www.exeldro.com\">Exeldro</a>"),
+		0, Qt::AlignLeft);
+	bottomLayout->addWidget(exportButton, 0, Qt::AlignRight);
+	bottomLayout->addWidget(importButton, 0, Qt::AlignRight);
+	bottomLayout->addWidget(matrixButton, 0, Qt::AlignRight);
+	bottomLayout->addWidget(deleteButton, 0, Qt::AlignRight);
 	bottomLayout->addWidget(closeButton, 0, Qt::AlignRight);
-
+	bottomLayout->setStretch(0, 1);
 	connect(deleteButton, &QPushButton::clicked,
 		[this]() { DeleteClicked(); });
 	connect(closeButton, &QPushButton::clicked, [this]() { close(); });
+	connect(exportButton, &QPushButton::clicked, [this]() {
+		const QString fileName = QFileDialog::getSaveFileName(
+			nullptr,
+			QString::fromUtf8(
+				obs_module_text("SaveTransitionTable")),
+			QString(), "JSON File (*.json)");
+		if (fileName.isEmpty())
+			return;
+		const auto fu = fileName.toUtf8();
+
+		const auto transitions_array = obs_data_array_create();
+		bool selection = false;
+		for (auto row = 2; row < mainLayout->rowCount(); row++) {
+			auto *item = mainLayout->itemAtPosition(row, 4);
+			if (!item)
+				continue;
+			auto *checkBox =
+				dynamic_cast<QCheckBox *>(item->widget());
+			if (!checkBox || !checkBox->isChecked())
+				continue;
+
+			item = mainLayout->itemAtPosition(row, 0);
+			auto *label = dynamic_cast<QLabel *>(item->widget());
+			if (!label)
+				continue;
+			string fromScene = label->text().toUtf8().constData();
+			if (fromScene == obs_module_text("Any"))
+				fromScene = "Any";
+			auto fs_it = transition_table.find(fromScene);
+			if (fs_it == transition_table.end())
+				continue;
+			item = mainLayout->itemAtPosition(row, 1);
+			label = dynamic_cast<QLabel *>(item->widget());
+			if (!label)
+				continue;
+			string toScene = label->text().toUtf8().constData();
+			if (toScene == obs_module_text("Any"))
+				toScene = "Any";
+			auto ts_it = fs_it->second.find(toScene);
+			if (ts_it == fs_it->second.end())
+				continue;
+			selection = true;
+			obs_data_t *transition = obs_data_create();
+			obs_data_set_string(transition, "from_scene",
+					    fromScene.c_str());
+			obs_data_set_string(transition, "to_scene",
+					    toScene.c_str());
+			obs_data_set_string(transition, "transition",
+					    ts_it->second.transition.c_str());
+			obs_data_set_int(transition, "duration",
+					 ts_it->second.duration);
+			obs_data_array_push_back(transitions_array, transition);
+			obs_data_release(transition);
+		}
+		if (!selection) {
+			for (const auto &it : transition_table) {
+				for (const auto &it2 : it.second) {
+					obs_data_t *transition =
+						obs_data_create();
+					obs_data_set_string(transition,
+							    "from_scene",
+							    it.first.c_str());
+					obs_data_set_string(transition,
+							    "to_scene",
+							    it2.first.c_str());
+					obs_data_set_string(
+						transition, "transition",
+						it2.second.transition.c_str());
+					obs_data_set_int(transition, "duration",
+							 it2.second.duration);
+					obs_data_array_push_back(
+						transitions_array, transition);
+					obs_data_release(transition);
+				}
+			}
+		}
+		obs_data_t *data = obs_data_create();
+		obs_data_set_array(data, "transitions", transitions_array);
+		obs_data_array_release(transitions_array);
+		obs_data_save_json(data, fu.constData());
+		obs_data_release(data);
+	});
+	connect(importButton, &QPushButton::clicked, [this]() {
+		const QString fileName = QFileDialog::getOpenFileName(
+			nullptr,
+			QString::fromUtf8(
+				obs_module_text("LoadTransitionTable")),
+			QString(), "JSON File (*.json)");
+		if (fileName.isEmpty())
+			return;
+		const auto fu = fileName.toUtf8();
+		obs_data_t *data =
+			obs_data_create_from_json_file(fu.constData());
+		load_transitions(data);
+		obs_data_release(data);
+		RefreshTable();
+		if (transition_table_enabled)
+			set_transition_overrides();
+	});
+	connect(matrixButton, &QPushButton::clicked, this,
+		&TransitionTableDialog::ShowMatrix);
 
 	vlayout = new QVBoxLayout;
 	vlayout->setContentsMargins(11, 11, 11, 11);
@@ -494,11 +624,16 @@ TransitionTableDialog::~TransitionTableDialog()
 
 void TransitionTableDialog::AddClicked()
 {
-	const auto fromScene = fromCombo->currentText();
-	const auto toScene = toCombo->currentText();
+	auto fromScene = fromCombo->currentText();
+	auto toScene = toCombo->currentText();
 	const auto transition = transitionCombo->currentText();
 	if (fromScene.isEmpty() || toScene.isEmpty() || transition.isEmpty())
 		return;
+	if (fromScene == QString::fromUtf8(obs_module_text("Any")))
+		fromScene = "Any";
+	if (toScene == QString::fromUtf8(obs_module_text("Any")))
+		toScene = "Any";
+
 	auto &t = transition_table[fromScene.toUtf8().constData()]
 				  [toScene.toUtf8().constData()];
 	t.transition = transition.toUtf8().constData();
@@ -523,6 +658,8 @@ void TransitionTableDialog::DeleteClicked()
 		if (!label)
 			continue;
 		string fromScene = label->text().toUtf8().constData();
+		if (fromScene == obs_module_text("Any"))
+			fromScene = "Any";
 		auto fs_it = transition_table.find(fromScene);
 		if (fs_it == transition_table.end())
 			continue;
@@ -531,6 +668,8 @@ void TransitionTableDialog::DeleteClicked()
 		if (!label)
 			continue;
 		string toScene = label->text().toUtf8().constData();
+		if (toScene == obs_module_text("Any"))
+			toScene = "Any";
 		auto ts_it = fs_it->second.find(toScene);
 		if (ts_it == fs_it->second.end())
 			continue;
@@ -559,8 +698,12 @@ void TransitionTableDialog::SelectAllChanged()
 
 void TransitionTableDialog::RefreshTable()
 {
-	const auto fromScene = fromCombo->currentText();
-	const auto toScene = toCombo->currentText();
+	auto fromScene = fromCombo->currentText();
+	auto toScene = toCombo->currentText();
+	if (fromScene == QString::fromUtf8(obs_module_text("Any")))
+		fromScene = "Any";
+	if (toScene == QString::fromUtf8(obs_module_text("Any")))
+		toScene = "Any";
 	for (auto row = mainLayout->rowCount() - 1; row >= 2; row--) {
 		for (auto col = mainLayout->columnCount() - 1; col >= 0;
 		     col--) {
@@ -590,6 +733,8 @@ void TransitionTableDialog::RefreshTable()
 				new QLabel(QString::fromUtf8(it.first.c_str()));
 			if (it.first == "Any") {
 				label->setProperty("themeID", "good");
+				label->setText(QString::fromUtf8(
+					obs_module_text("Any")));
 			} else {
 				auto scene = obs_get_source_by_name(
 					it.first.c_str());
@@ -604,6 +749,8 @@ void TransitionTableDialog::RefreshTable()
 				QString::fromUtf8(it2.first.c_str()));
 			if (it2.first == "Any") {
 				label->setProperty("themeID", "good");
+				label->setText(QString::fromUtf8(
+					obs_module_text("Any")));
 			} else {
 				auto scene = obs_get_source_by_name(
 					it2.first.c_str());
@@ -672,3 +819,78 @@ void TransitionTableDialog::mouseDoubleClickEvent(QMouseEvent *event)
 	fromCombo->setCurrentText(from);
 	toCombo->setCurrentText(to);
 }
+
+void TransitionTableDialog::ShowMatrix()
+{
+	const auto md = new QDialog(this);
+	md->setWindowTitle(
+		QString::fromUtf8(obs_module_text("TransitionMatrix")));
+	md->setAttribute(Qt::WA_DeleteOnClose);
+	md->setSizeGripEnabled(true);
+
+	std::list<std::string> scenes;
+	for (const auto &it : transition_table) {
+		if (it.first != "Any")
+			scenes.push_back(it.first);
+		for (const auto &it2 : it.second) {
+			if (it2.first != "Any")
+				scenes.push_back(it2.first);
+		}
+	}
+	scenes.sort();
+	scenes.unique();
+	scenes.push_front("Any");
+	const int s = (int)scenes.size();
+	const auto w = new QTableWidget(s, s);
+	int i = 0;
+	for (const auto &it : scenes) {
+		w->setHorizontalHeaderItem(
+			i, new QTableWidgetItem(
+				   QString::fromUtf8(
+					   it == "Any" ? obs_module_text("Any")
+						       : it.c_str()),
+				   QTableWidgetItem::ItemType::Type));
+		w->setVerticalHeaderItem(
+			i, new QTableWidgetItem(
+				   QString::fromUtf8(
+					   it == "Any" ? obs_module_text("Any")
+						       : it.c_str()),
+				   QTableWidgetItem::ItemType::Type));
+		i++;
+	}
+	int row = 0;
+	for (const auto &it : scenes) {
+		auto f1 = transition_table.find(it);
+		int column = 0;
+		for (const auto &it2 : scenes) {
+			string t;
+			if (f1 != transition_table.end()) {
+				auto f2 = f1->second.find(it2);
+				if (f2 != f1->second.end()) {
+					t = f2->second.transition;
+				}
+			}
+			w->setCellWidget(
+				row, column,
+				new QLabel(QString::fromUtf8(t.c_str())));
+			column++;
+		}
+		row++;
+	}
+	const auto m = new QVBoxLayout;
+	m->addWidget(w);
+
+	QPushButton *closeButton =
+		new QPushButton(QString::fromUtf8(obs_module_text("Close")));
+	connect(closeButton, &QPushButton::clicked, [md]() { md->close(); });
+	QHBoxLayout *bottomLayout = new QHBoxLayout;
+	bottomLayout->addWidget(
+		new QLabel(
+			"<a href=\"https://obsproject.com/forum/resources/transition-table.1174/\">Transition Table</a> (" PROJECT_VERSION
+			") by <a href=\"https://www.exeldro.com\">Exeldro</a>"),
+		0, Qt::AlignLeft);
+	bottomLayout->addWidget(closeButton, 0, Qt::AlignRight);
+	m->addLayout(bottomLayout);
+	md->setLayout(m);
+	md->exec();
+}
