@@ -8,7 +8,7 @@
 #include <QGridLayout>
 #include <QSpinBox>
 
-#include "obs-frontend-api.h"
+#include <obs-frontend-api.h>
 
 class TransitionTableDialog : public QDialog {
 	Q_OBJECT
@@ -29,6 +29,7 @@ public:
 	~TransitionTableDialog();
 public slots:
 	void RefreshTable();
+	void ShowMatrix();
 
 protected:
 	virtual void mouseDoubleClickEvent(QMouseEvent *event) override;
